# library(latex2exp) # annotate plot with formula
library(MASS)      # fractions
library(plyr)      # mapvalues
library(ggplot2); theme_set(theme_bw())
library(gridExtra) # plot layout
library(cowplot)   # draw_plot_label
library(ggpubr)    # as_ggplot

# load data ----------------------------------------
param_true <- rbind(c(1/3,1/3,200,0.4),
                    c(3,1/3,200,0.4),
                    c(1/3,3,200,0.4))

process_data <- readRDS('gex_process_data.rds')
dist_data <- readRDS('gex_dist_data.rds')

# reorganize data ----------------------------------------
min_max_t <- min(sapply(process_data, function(x){
  tail(x$time_axis, 1)
}))
process_data <- lapply(process_data, function(x){
  idx_sel <- which(x$time_axis <= min_max_t)
  return(list(time_axis = x$time_axis[idx_sel],
              realization = x$realization[idx_sel, ]))
})

process_label <- Reduce(c, sapply(1:length(process_data), function(iter){
  rep(names(process_data)[iter], length(process_data[[iter]][['time_axis']]))
}))
time_axis_dat <- Reduce(c, sapply(process_data, function(x){
  x$time_axis
}))
n_mRNA_dat <- Reduce(c, sapply(process_data, function(x){
  x$realization[, 2]
}))

process_dat <- data.frame(time_axis = time_axis_dat, n_mRNA = n_mRNA_dat,
                          param_group = process_label)


dist_label <- Reduce(c, sapply(1:length(dist_data), function(iter){
  rep(names(dist_data)[iter], length(dist_data[[iter]]))
}))
dist_dat <- data.frame(n_mRNA = unlist(dist_data), param_group = dist_label)

# create labeller function ----------------------------------------
# create character strings that can be interpreted as plotmath expressions:
# https://stat.ethz.ch/R-manual/R-devel/library/grDevices/html/plotmath.html
labeller_fun <- function(value){
  value <- as.integer(value)
  # k_a <- round(param_true[value, 1], 2)
  # k_i <- round(param_true[value, 2], 2)
  # s_m <- round(param_true[value, 3], 2)
  get_frac_rep <- function(x, over = TRUE){
    if(over){
      x <- strsplit(attr(fractions(x), 'fracs'), '/')[[1]]
      if(length(x) == 2){
        return(paste0('frac(', x[1], ',', x[2], ')'))
      }else{
        return(x)
      }
    }else{
      return(attr(fractions(x), 'fracs'))
    }
  }
  k_a <- get_frac_rep(param_true[value, 1], over = FALSE)
  k_i <- get_frac_rep(param_true[value, 2], over = FALSE)
  plot_title <-
    paste0('list(', 'k[a]==', k_a, ',~~k[i]==', k_i, ')')
  plot_title
}

# global_labeller <- labeller(`1` = labeller_fun('1'),
#                             `2` = labeller_fun('2'),
#                             `3` = labeller_fun('3'))

process_dat$param_group <- factor(process_dat$param_group)
process_dat$param_group <- mapvalues(process_dat$param_group,
                                     from = c('1', '2', '3'),
                                     to = c(labeller_fun('1'),
                                            labeller_fun('2'),
                                            labeller_fun('3')))
dist_dat$param_group <- factor(dist_dat$param_group)
dist_dat$param_group <- mapvalues(dist_dat$param_group,
                                  from = c('1', '2', '3'),
                                  to = c(labeller_fun('1'),
                                         labeller_fun('2'),
                                         labeller_fun('3')))

# plotting ----------------------------------------
process_plot <- ggplot(data = process_dat, aes(x = time_axis, y = n_mRNA)) +
  geom_line() +
  facet_wrap(~param_group, labeller = label_parsed) +
  xlab('time') + ylab('mRNA copy number') +
  theme(strip.text = element_text(size=15),
        axis.text = element_text(size = 12),
        axis.title = element_text(size = 15),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())

dist_plot <- ggplot(data = dist_dat, aes(x = n_mRNA)) +
  geom_histogram(aes(y = stat(density))) +
  facet_wrap(~param_group, labeller = label_parsed) +
  xlab('mRNA copy number') + ylab('density') +
  theme(strip.text = element_text(size=15),
        axis.text = element_text(size = 12),
        axis.title = element_text(size = 15),
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank())


gt <- arrangeGrob(process_plot, dist_plot, nrow = 2,
                  heights = c(1, 1))
final_plot <- as_ggplot(gt) +
  draw_plot_label(label = c('b', 'c'), x = c(0, 0), y = c(1, 0.5))

pdf('gex_process_dist_plot.pdf', height = 6, width = 9)
print(final_plot)
dev.off()















